import lib-env.glsl
import lib-normal.glsl
import lib-alpha.glsl
import lib-pbr.glsl
import lib-vectors.glsl


//: state cull_face off

//-------------------------- Diffuse Map --------------------------//

//: param auto channel_diffuse
uniform sampler2D diffuseMapSampler;

//: param auto channel_opacity
uniform sampler2D opacitySampler;

//-------- Normal Map ---------------------------------------------------//

//: param auto channel_user0
uniform sampler2D envmapmaskSampler;

//: param auto channel_specular
uniform sampler2D specularSampler;

//: param auto channel_glossiness
uniform sampler2D glossinessSampler;

//: param auto channel_metallic
uniform sampler2D metalllicSampler;

//: param auto channel_user1
uniform sampler2D colormaskSampler;

//: param custom { "default": 1, "label": "Diffuse Color", "widget": "color" }
uniform vec3 fColor;

//-------- Envmap Parameters ---------------------------------------------------//

//: param custom { "default": true, "label": "Enable Envmap", "group": "Envmap" }
uniform bool bUseEnvmap;

//: param custom { "default": true, "label": "Use Specular Channel", "group": "Envmap" }
uniform bool bUseSpecular;

//: param custom { "default": 1, "label": "Envmap Tint", "widget": "color", "group": "Envmap"}
uniform vec3 fEnvmapColor;

//: param custom { "default": 0, "label": "Envmap Fresnel", "min": 0.0, "max": 1.0, "group": "Envmap" }
uniform float fEnvmapFresnel;

//-------- Specular Parameters ---------------------------------------------------//

//: param custom { "default": [0.25, 0.5, 1.0], "label": "Fresnel Ranges", "group": "Specular" }
uniform vec3 fFresnelRanges;

//: param custom { "default": 1, "label": "Specular Tint", "widget": "color", "group": "Specular" }
uniform vec3 fSpecularColor;

//: param custom { "default": 1.0, "label": "Specular Boost", "min": 0.0, "max": 16.0, "group": "Specular" }
uniform float fSpecularScale;

//: param custom { "default": 5.0, "label": "Specular Glossiness", "min": 1.0, "max": 512.0, "group": "Specular" }
uniform float fSpecularExponent;

//: param custom { "default": false, "label": "Use Glossiness Channel", "group": "Specular" }
uniform bool bUseGlossiness;

//: param custom { "default": false, "label": "Use Metallic Channel", "group": "Specular" }
uniform bool bUseMetallic;

//: param custom { "default": 1.0, "label": "Metallic Boost", "min": 1.0, "max": 512.0, "group": "Specular" }
uniform float fDiffuseBoost;

//-------- Ambient ---------------------------------------------------//

//: param auto main_light
uniform vec4 light_main;

float saturate(float v)
{
  return clamp(v, 0.0, 1.0);
}

vec3 saturate(vec3 v)
{
  return clamp(v, 0.0, 1.0);
}

float fresnelRanges(vec3 normal, vec3 eye, vec3 ranges){
	float result;
	float f = 1 - saturate(dot(normal, eye));
	f *= f;

	if(f > 0.5f) 
		result = mix(ranges.g, ranges.b, (2*f)-1);
	else 
		result = mix(ranges.r, ranges.g, (2*f));

	return result;
}

vec4 shade(V2F inputs)
{

	LocalVectors vectors = computeLocalFrame(inputs);

	vec3 normalWS = vectors.normal;

	vec3 V = vectors.eye;

	vec4 albedo = texture(diffuseMapSampler, inputs.tex_coord);
	albedo.rgb += DEFAULT_BASE_COLOR * (1.0 - albedo.a);

	float flEnvmapMask = getRoughness(envmapmaskSampler, inputs.tex_coord);;
	if (bUseSpecular) {
			flEnvmapMask = getRoughness(specularSampler, inputs.tex_coord);
	}
	float flSpecularMask = getRoughness(specularSampler, inputs.tex_coord);
	float flSpecularExponent = getRoughness(glossinessSampler, inputs.tex_coord);
	float flColorMask = getRoughness(colormaskSampler, inputs.tex_coord);

	float VdotN = saturate(dot(V, normalWS));

	// Compute local vectors and cos of some useful angles
	float ndl = max(0.0, dot(vectors.normal, light_main.xyz));
	float ndv = max(0.0, dot(vectors.normal, vectors.eye));
	float ndh = max(0.0, dot(vectors.normal, normalize(light_main.xyz + vectors.eye)));

	vec3 diffuseColor = vec3(0.5); // global diffuse color just so we can see something by default ( gets overwritten by albedo texturemap if enabled )
	diffuseColor = mix(albedo.rgb, albedo.rgb * fColor, flColorMask); // assign the diffuse texture to our global diffuse color

	vec3 ambient = envIrradiance(normalWS);

	vec3 finalDiffuse = vec3(0.0);
	vec3 finalSpecular = vec3(0.0);
	vec3 finalEnvmap = vec3(0.0);

	//finalDiffuse += ambient;
	vec3 final;

	vec3 L = normalize(light_main.xyz);

	float halfLambert = ndl * 0.5 + 0.5;

	vec3 diffuseLight = vec3(0.0);
	diffuseLight += halfLambert;
	finalDiffuse += diffuseLight;

	vec3 R = reflect( vectors.eye, vectors.normal ); // No half-vector so this is consistent in look with ps2.0
	float RdotL = saturate(dot( L, -R ));
	vec3 specularLighting = vec3(0.0);

	float exponent = fSpecularExponent;

	float flFresnel = fresnelRanges(vectors.normal, vectors.eye, fFresnelRanges);

	if (bUseGlossiness) {
		exponent = 1.0 - flSpecularExponent + 150.0 * flSpecularExponent;
	}

	float flSpecularIntensity = ndl * pow(ndh, exponent);
	specularLighting = vec3(flSpecularIntensity);

	finalSpecular += specularLighting;

	final = (diffuseColor * ambient);

	finalEnvmap = envSampleLOD(-R, 2);

	vec3 cEnvmap = finalEnvmap * fEnvmapColor;
	cEnvmap *= flEnvmapMask;

	vec3 cSpecular = finalSpecular * fSpecularScale * flFresnel;
	cSpecular *= flSpecularMask;
	cSpecular *= fSpecularColor;

	vec3 cMetalSpecular = cSpecular * albedo.rgb * fDiffuseBoost;
	vec3 cMetalEnvmap = cEnvmap;

	if (bUseMetallic) {
		cSpecular = mix(cSpecular, cMetalSpecular, getRoughness(metalllicSampler, inputs.tex_coord));
		cMetalEnvmap = mix(cEnvmap, cEnvmap * albedo.rgb * fDiffuseBoost, getRoughness(metalllicSampler, inputs.tex_coord));
	}

	cEnvmap = mix(cEnvmap, cMetalEnvmap * flFresnel, fEnvmapFresnel);

	final += cSpecular;

	if (bUseEnvmap) {
		final += cEnvmap;
	}

	final *= getShadowFactor();

	return vec4(final,1);

}

void shadeShadow(V2F inputs)
{
}
